// dispatch_iterate.m -- iterate over an array doing work.

// gcc -std=c99 -g -Wall -o dispatch_iterate dispatch_iterate.m

#import <dispatch/dispatch.h>  // for GCD
#import <stdio.h> // for printf()


enum {
    kNumberCount = 200000000,
    kStride = 1000,
    kCountCount = 10,
    kSleepTime = 0
};

static int numbers[kNumberCount];
static int results[kNumberCount];


static int Work (int *data, int index) {
    return data[index] * 2;
} // Work


int main (void) {
    // Initialize numbers
    printf ("STARTING!\n");

    for (int i = 0; i < kNumberCount; i++) {
        numbers[i] = i;
    }

    printf ("INIT!\n");
    sleep (kSleepTime);
    printf ("STARTING iterate!\n");

    // Iterative
    for (int count = 0; count < kCountCount; count++) {
        for (int i = 0; i < kNumberCount; i++) {
            results[i] = Work (numbers, i);
        }
    }

    printf ("DONE iterate!\n");
    sleep (kSleepTime);
    printf ("STARTING dispatch!\n");

    dispatch_queue_t queue =
        dispatch_get_global_queue (DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);

    for (int count = 0; count < kCountCount; count++) {
        dispatch_apply (kNumberCount, queue, ^(size_t index) {
                results[index] = Work (numbers, index);
            });
    }
        
    printf ("DONE dispatch!\n");
    sleep (kSleepTime);
    printf ("STARTING dispatch-stride\n");

    for (int count = 0; count < kCountCount; count++) {
        dispatch_apply (kNumberCount / kStride, queue, ^(size_t index) {
                size_t jindex = index * kStride;
                size_t jStop = MIN(jindex + kStride, kNumberCount);
                while (jindex < jStop) {
                    results[jindex] = Work (numbers, jindex);
                }
            });
    }

    printf ("DONE dispatch-stride\n");

    return 0;

} // main
